"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

'From Smalltalk/X, Version:2.10.4 on 24-feb-1995 at 5:09:20 am'!

Model subclass:#SystemBrowser
	 instanceVariableNames:'currentClass currentMethodCategory currentMethod currentSelector
		showInstance actualClasslastMethodCategory aspect lockUpdates
		autoSearch myLabel acceptClass'
	 classVariableNames:'CheckForInstancesWhenRemovingClasses'
	 poolDictionaries:''
	 category:'Interface-Browsers'
!

!SystemBrowser class methodsFor:'initialization'!

initialize
    "Browser configuration;
     (values can be changed from your private startup file)"

    "
     setting this to false, the removeClass function will remove
     classes WITHOUT checking for instances. Otherwise,
     it will check and let you confirm in case there are instances.
     Checking for instances may be a bit time consuming, though.
     The default is true - therefore, it will check
    "
    CheckForInstancesWhenRemovingClasses := true

    "
     CheckForInstancesWhenRemovingClasses := true
     CheckForInstancesWhenRemovingClasses := false

     Browser initialize
    "
! !

!SystemBrowser class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

version
"
$Header: /cvs/stx/stx/libtool/SystemBrowser.st,v 1.27 1995-03-06 21:25:58 claus Exp $
"
!

documentation
"
    this class implements all kinds of class browsers.
    Typically, it is started with 'SystemBrowser open', but there are many other 
    startup messages, to launch special browsers.
    See the categories 'startup' and 'special search startup' in the classes
    protocol.

    Alse, see the extra document 'doc/misc/sbrowser.doc' or the HTML online doc
    for how to use the browser.

    written winter 89 by claus
"
! !

!SystemBrowser class methodsFor:'instance creation'!

openOnDisplay:aDisplay
    "launch a standard browser on another display.
     Does not work currently - still being developped."

    ^ self newWithLabel:(BrowserView classResources string:'System Browser')
	     setupBlock:[:browser | browser setupForAll]
		     on:aDisplay

    "
     SystemBrowser openOnDisplay:(XWorkstation new initializeFor:'porty:0')
    "
!

open
    "launch a standard browser"

    ^ self openOnDisplay:Display

    "SystemBrowser open"
! !

!SystemBrowser class methodsFor:'startup'!

browseMethods:aList title:aString
    "launch a browser for an explicit list of class/selectors.
     Each entry in the list must consist of the classes name and the selector,
     separated by spaces. For class methods, the string 'class' must be
     appended to the classname."

    (aList size == 0) ifTrue:[
	self showNoneFound:aString.
	^ nil
    ].
    aList sort.
    ^ self newWithLabel:aString
	     setupBlock:[:browser | browser setupForList:aList]

    "
     SystemBrowser browseMethods:#('Object printOn:' 
				   'Collection add:')
			   title:'some methods'
    "
    "
     SystemBrowser browseMethods:#('Behavior new:' 
				   'Setclass new:')
			   title:'some new: methods'
    "
!

browseMethodsIn:aCollectionOfClasses inst:wantInst class:wantClass where:aBlock title:title
    "launch a browser for all instance- (if wantInst is true) and/or
     classmethods (if wantClass is true) from classes in aCollectionOfClasses,
     where aBlock evaluates to true.
     The block is called with 3 arguments, class, method and seelctor."

    |list|

    "
     since this may take a long time, lower my priority ...
    "
    Processor activeProcess withLowerPriorityDo:[
	|checkedClasses checkBlock|

	checkedClasses := IdentitySet new.
	list := OrderedCollection new.

	checkBlock := [:cls |
	    |methodArray selectorArray| 

	    (checkedClasses includes:cls) ifFalse:[
		methodArray := cls methodArray.
		selectorArray := cls selectorArray.

		1 to:methodArray size do:[:index |
		    |method sel|

		    method := methodArray at:index.
		    sel := selectorArray at:index.
		    (aBlock value:cls value:method value:sel) ifTrue:[
			list add:(cls name , ' ' , sel)
		    ]
		].
		checkedClasses add:cls.
	    ]
	].

	aCollectionOfClasses do:[:aClass |
	    "
	     output disabled - it slows down things too much (when searching for
	     implementors or senders)
	    "
"/            Transcript show:'searching '; show:aClass name; showCr:' ...'; endEntry.
	    wantInst ifTrue:[checkBlock value:aClass].
	    wantClass ifTrue:[checkBlock value:(aClass class)].
	    Processor yield
	]
    ].

    ^ self browseMethods:list title:title
!

browseMethodsIn:aCollectionOfClasses where:aBlock title:title
    "launch a browser for all instance- and classmethods from 
     all classes in aCollectionOfClasses where aBlock evaluates to true.
     The block is called with 3 arguments, class, method and seelctor."

    ^ self browseMethodsIn:aCollectionOfClasses inst:true class:true where:aBlock title:title

!

browseClassCategory:aClassCategory
    "launch a browser for all classes under aCategory"

    ^ self newWithLabel:aClassCategory
	     setupBlock:[:browser | browser setupForClassCategory:aClassCategory]

    "SystemBrowser browseClassCategory:'Kernel-Objects'"
!

browseFullClasses
    "launch a browser showing all methods at once"

    ^ self newWithLabel:'Full Class Browser'
	     setupBlock:[:browser | browser setupForFullClass]

    "SystemBrowser browseFullClasses"
!

browseClass:aClass
    "launch a browser for aClass"

    ^ self newWithLabel:aClass name
	     setupBlock:[:browser | browser setupForClass:aClass]

    "SystemBrowser browseClass:Object"
!

browseClass:aClass selector:selector
    "launch a browser for the method at selector in aClass"

    ^ self 
	newWithLabel:(aClass name , ' ' , selector , ' ' , selector)
	setupBlock:[:browser | browser setupForClass:aClass selector:selector]

    "
     SystemBrowser browseClass:Object selector:#printString
    "
!

browseClassHierarchy:aClass
    "launch a browser for aClass and all its superclasses.
     this is different from the fullProtocol browser."

    ^ self newWithLabel:(aClass name , '-' , 'hierarchy')
	     setupBlock:[:browser | browser setupForClassHierarchy:aClass]

    "
     SystemBrowser browseClassHierarchy:Number
    "
!

browseFullClassProtocol:aClass
    "launch a browser for aClasses full protocol.
     This is different from hierarchy browsing."

    ^ self newWithLabel:(aClass name , '-' , 'full protocol')
	     setupBlock:[:browser | browser setupForFullClassProtocol:aClass]

    "
     SystemBrowser browseFullClassProtocol:Number
    "
!

browseClasses:aList title:title
    "launch a browser for all classes in aList"

    ^ self newWithLabel:title
	     setupBlock:[:browser | browser setupForClassList:aList]

    "
     SystemBrowser browseClasses:(Array with:Object
					with:Float)
			   title:'two classes'
    "
!

browseClass:aClass methodCategory:aCategory
    "launch a browser for all methods under aCategory in aClass"

    ^ self newWithLabel:(aClass name , ' ' , aCategory)
	  setupBlock:[:browser | browser setupForClass:aClass methodCategory:aCategory]

    "SystemBrowser browseClass:String methodCategory:'copying'"
!

browseMethodCategory:aCategory
    "launch a browser for all methods where category = aCategory"

    |searchBlock|

    aCategory includesMatchCharacters ifTrue:[
	searchBlock := [:c :m :s | aCategory match:m category].
    ] ifFalse:[
	searchBlock := [:c :m :s | m category = aCategory]
    ].

    self browseMethodsWhere:searchBlock title:('all methods with category of ' , aCategory)

    "
     SystemBrowser browseMethodCategory:'printing & storing'
     SystemBrowser browseMethodCategory:'print*'
    "
!

browseAllSelect:aBlock
    "launch a browser for all methods where aBlock returns true.
     The block is called with 3 arguments, class, method and seelctor."

    ^ self browseMethodsWhere:aBlock title:'selected messages'

    "
     SystemBrowser browseAllSelect:[:aClass :aMethod :selector | selector numArgs == 3]
    "
!

browseMethodsWhere:aBlock title:title
    "launch a browser for all methods where aBlock returns true.
     The block is called with 3 arguments, class, method and seelctor."

    ^ self browseMethodsIn:(Smalltalk allClasses) where:aBlock title:title
!

browseMethodsOf:aClass where:aBlock title:title
    "launch a browser for all instance- and classmethods in aClass 
     where aBlock evaluates to true.
     The block is called with 3 arguments, class, method and seelctor."

    ^ self browseMethodsIn:(Array with:aClass) where:aBlock title:title
!

browseMethodsFrom:aClass where:aBlock title:title
    "launch a browser for all instance- and classmethods in aClass
     and all its subclasses where aBlock evaluates to true.
     The block is called with 3 arguments, class, method and seelctor."

    ^ self browseMethodsIn:(aClass withAllSubclasses) where:aBlock title:title
!

browseInstMethodsOf:aClass where:aBlock title:title
    "launch a browser for all instance methods in aClass
     where aBlock evaluates to true"

    ^ self browseMethodsIn:(Array with:aClass) inst:true class:false where:aBlock title:title
!

browseInstMethodsIn:aCollectionOfClasses where:aBlock title:title
    "launch a browser for all instance methods of all classes in
     aCollectionOfClasses where aBlock evaluates to true"

    ^ self browseMethodsIn:aCollectionOfClasses inst:true class:false 
		     where:aBlock title:title
!

browseInstMethodsFrom:aClass where:aBlock title:title
    "launch a browser for all instance methods in aClass and all subclasses
     where aBlock evaluates to true"

    ^ self browseMethodsIn:(aClass withAllSubclasses) inst:true class:false where:aBlock title:title
! !

!SystemBrowser class methodsFor:'special search startup'!

browseAllCallsOn:aSelectorString in:aCollectionOfClasses title:title
    "launch a browser for all senders of aSelector in aCollectionOfClasses"

    |sel browser searchBlock|

    ((aSelectorString ~= '*') and:[aSelectorString includesMatchCharacters]) ifTrue:[
	"a matchString"
	searchBlock := [:lits |
			    |found|

			    found := false.
			    lits notNil ifTrue:[
				lits do:[:aLiteral |
				    found ifFalse:[
					(aLiteral isMemberOf:Symbol) ifTrue:[
					    found := (aSelectorString match:aLiteral)
					]
				    ]
				]
			    ].
			    found
		       ].
	browser := self browseMethodsIn:aCollectionOfClasses
				  where:[:class :method :s | searchBlock value:(method literals)]
				  title:title
    ] ifFalse:[
	sel := aSelectorString asSymbolIfInterned.
	sel isNil ifTrue:[
"
	    Transcript showCr:'none found.'.
"
	    self showNoneFound:title.
	    ^ nil
	].

	browser := self browseMethodsIn:aCollectionOfClasses
				  where:[:class :method :s | method sends:sel]
				  title:title
    ].

    browser notNil ifTrue:[
	|s|

	"
	 kludge for now, if its a multipart selector,
	 no easy search is (as yet) possible
	"
	s := aSelectorString.
	(s includes:$:) ifTrue:[
	    s := s copyTo:(s indexOf:$:)
	].
	browser autoSearch:s 
    ].
    ^ browser
!

browseImplementorsOf:aSelectorString
    "launch a browser for all implementors of aSelector"

    ^ self browseImplementorsOf:aSelectorString
			     in:(Smalltalk allClasses)
			  title:('implementors of: ' , aSelectorString)

    "
     SystemBrowser browseImplementorsOf:#+
    "
!

browseImplementorsOf:aSelectorString in:aCollectionOfClasses title:title
    "launch a browser for all implementors of aSelector in
     the classes contained in aCollectionOfClasses and its metaclasses"

    |list sel|

    list := OrderedCollection new.

    ((aSelectorString ~= '*') and:[aSelectorString includesMatchCharacters]) ifTrue:[
	"a matchString"

	aCollectionOfClasses do:[:aClass |
	    aClass selectorArray do:[:aSelector |
		(aSelectorString match:aSelector) ifTrue:[
		    list add:(aClass name , ' ' , aSelector)
		]
	    ].
	    aClass class selectorArray do:[:aSelector |
		(aSelectorString match:aSelector) ifTrue:[
		    list add:(aClass name , 'class ' , aSelector)
		]
	    ]
	]
    ] ifFalse:[
	"can do a faster search"

	sel := aSelectorString asSymbolIfInterned.
	sel isNil ifTrue:[
	    self showNoneFound:title.
	    ^ nil
	].

	aCollectionOfClasses do:[:aClass |
	    (aClass implements:sel) ifTrue:[
		list add:(aClass name , ' ' , aSelectorString)
	    ].
	    (aClass class implements:sel) ifTrue:[
		list add:(aClass name , 'class ' , aSelectorString)
	    ]
	]
    ].
    ^ self browseMethods:list title:title

    "
     SystemBrowser browseImplementorsOf:#+
				     in:(Array with:Number
					       with:Float
					       with:SmallInteger)
				  title:'some implementors of +'
    "
!

browseAllCallsOn:aSelectorString
    "launch a browser for all senders of aSelector"

    ^ self browseAllCallsOn:aSelectorString 
			 in:(Smalltalk allClasses)
		      title:('senders of ' , aSelectorString)

    "
     SystemBrowser browseAllCallsOn:#+
    "
!

browseCallsOn:aSelectorString under:aClass
    "launch a browser for all senders of aSelector in aClass and subclasses"

    ^ self browseAllCallsOn:aSelectorString
			 in:(aClass withAllSubclasses)
		      title:('senders of: ' , 
			     aSelectorString , 
			     ' (in or below ' , aClass name , ')')

    "
     SystemBrowser browseAllCallsOn:#+ under:Number
    "
!

browseImplementorsOf:aSelectorString under:aClass
    "launch a browser for all implementors of aSelector in aClass
     and its subclasses"

    ^ self browseImplementorsOf:aSelectorString
			     in:(aClass withAllSubclasses)
			  title:('implementors of: ' , 
				 aSelectorString , 
				 ' (in or below ' , aClass name , ')')

    "
     SystemBrowser browseImplementorsOf:#+ under:Integer
    "
!

browseForSymbol:aSymbol title:title warnIfNone:doWarn
    "launch a browser for all methods referencing aSymbol"

    |browser searchBlock sym|

    (aSymbol includesMatchCharacters) ifTrue:[
	"a matchString"
	searchBlock := [:lits |
			    |found|

			    found := false.
			    lits notNil ifTrue:[
				lits do:[:aLiteral |
				    found ifFalse:[
					(aLiteral isMemberOf:Symbol) ifTrue:[
					    found := (aSymbol match:aLiteral)
					]
				    ]
				]
			    ].
			    found
		       ].
    ] ifFalse:[
	"
	 can do a faster search
	"
	sym := aSymbol asSymbolIfInterned.
	sym isNil ifTrue:[
	    self showNoneFound:title.
	    ^ nil
	].

	searchBlock := [:lits |
			    |found|

			    found := false.
			    lits notNil ifTrue:[
				lits do:[:aLiteral |
				    found ifFalse:[
					(aLiteral isMemberOf:Symbol) ifTrue:[
					    found := (sym == aLiteral)
					]
				    ]
				]
			    ].
			    found
		       ].
    ].
    browser := self browseMethodsWhere:[:c :m :s | searchBlock value:(m literals)] title:title.
    browser notNil ifTrue:[
	browser autoSearch:aSymbol
    ].
    ^ browser
!

filterToSearchInstRefsTo:varName modificationsOnly:modsOnly
    "return a block to search for instvar accesses"

    |searchBlock|

    searchBlock := [:c :m :s |
	|src result parser instvars needMatch|

	needMatch := varName includesMatchCharacters.

	src := m source.
	src isNil ifTrue:[
	    result := false
	] ifFalse:[
	    needMatch ifFalse:[
		"
		 before doing a slow parse, quickly scan the
		 methods source for the variables name ...
		"
		result := (src findString:varName) ~~ 0.
	    ] ifTrue:[
		result := true.
	    ].
	    result ifTrue:[
		result := false.
		parser := Parser parseMethod:src in:c warnings:false.
		parser notNil ifTrue:[
		    modsOnly ifTrue:[
			instvars := parser modifiedInstVars
		    ] ifFalse:[
			instvars := parser usedInstVars
		    ].
		    instvars notNil ifTrue:[
			needMatch ifTrue:[
			    instvars do:[:iv |
				(varName match:iv) ifTrue:[result := true]
			    ]
			] ifFalse:[
			    result := instvars includes:varName
			]
		    ]
		]
	    ].
	].
	Processor yield.
	result
    ].
    ^ searchBlock
!

browseForSymbol:aSymbol
    "launch a browser for all methods referencing aSymbol"

    ^ self browseForSymbol:aSymbol title:('users of ' , aSymbol) warnIfNone:true
!

browseReferendsOf:aGlobalName warnIfNone:doWarn
    "launch a browser for all methods referencing a global
     named aGlobalName.
    "

    ^ self browseForSymbol:aGlobalName title:('users of: ' , aGlobalName) warnIfNone:doWarn 

!

browseReferendsOf:aGlobalName
    "launch a browser for all methods referencing a global
     named aGlobalName.
    "

    ^ self browseReferendsOf:aGlobalName warnIfNone:true 

   "
    Browser browseReferendsOf:#Transcript
   "

!

browseUsesOf:aClass
    |dict owners offsets
     sz  "{ Class: SmallInteger }"
     n   "{ Class: SmallInteger }"
     removeSet newDict|

    owners := ObjectMemory whoReferencesInstancesOf:aClass.

    "
     collect set of offsets in dict; key is class
    "
    dict := IdentityDictionary new.
    owners do:[:someObject |
	|cls create|

	someObject isContext ifFalse:[
	    "
	     someObject refers to an instance of aClass;
	     find out, which instVar(s)
	    "
	    cls := someObject class.
	    cls ~~ Array ifTrue:[
		n := cls instSize.
		create := [|s| s := Set new. dict at:cls put:s. s].

		1 to:n do:[:i |
		    |ref|

		    ref := someObject instVarAt:i.
		    (ref isMemberOf:aClass) ifTrue:[
			offsets := dict at:cls ifAbsent:create.
			offsets add:i.
		    ]
		].
		cls isVariable ifTrue:[
		    cls isPointers ifTrue:[
			| idx "{ Class: SmallInteger }" |

			sz := someObject basicSize.
			idx := 1.
			[idx <= sz] whileTrue:[
			    |ref|

			    ref := someObject basicAt:idx.
			    (ref isMemberOf:aClass) ifTrue:[
				offsets := dict at:cls ifAbsent:create.
				offsets add:0.
				idx := sz
			    ].
			    idx := idx + 1
			]
		    ]        
		]
	    ]
	]
    ].

    "
     merge with superclass refs
    "
    dict keysAndValuesDo:[:cls :set |
	cls allSuperclasses do:[:aSuperclass |
	    |superSet|

	    superSet := dict at:aSuperclass ifAbsent:[].
	    superSet notNil ifTrue:[
		|removeSet|

		superSet := dict at:aSuperclass.
		removeSet := Set new.
		set do:[:offset |
		    (superSet includes:offset) ifTrue:[
			removeSet add:offset
		    ]
		].
		set removeAll:removeSet
	    ]
	]
    ].

    "
     remove empty ones
    "
    removeSet := Set new.
    dict keysAndValuesDo:[:cls :set |
	set isEmpty ifTrue:[
	    removeSet add:cls
	]
    ].
    removeSet do:[:cls |
	dict removeKey:cls
    ].

    "
     replace the indices by real names
    "
    newDict := IdentityDictionary new.
    dict keysAndValuesDo:[:cls :set |
	|newSet names|

	names := cls allInstVarNames.
	newSet := set collect:[:index | 
		index == 0 ifTrue:['*indexed*'] ifFalse:[names at:index].
	].
	newDict at:cls put:newSet
    ].

    newDict inspect


!

browseForString:aString in:aCollectionOfClasses
    "launch a browser for all methods in aCollectionOfClasses  
     containing a string in their source.
     This may be slow, since source-code has to be scanned."

    |browser searchBlock title s|

    title := 'methods containing: ' , aString displayString.

    (aString includesMatchCharacters) ifTrue:[
	s := '*' , aString , '*'.
	"a matchString"
	searchBlock := [:source | s match:source]
    ] ifFalse:[
	searchBlock := [:source | (source findString:aString) ~~ 0]
    ].
    browser := self browseMethodsIn:aCollectionOfClasses 
			      where:[:c :m :s | searchBlock value:(m source)] 
			      title:title.

    browser notNil ifTrue:[
	browser autoSearch:aString
    ].
    ^ browser

    "
     SystemBrowser browseForString:'all' in:(Array with:Object)
     SystemBrowser browseForString:'should' in:(Array with:Object)
     SystemBrowser browseForString:'[eE]rror' in:(Array with:Object)
    "

!

browseForString:aString
    "launch a browser for all methods containing a string in their source.
     This may be slow, since source-code has to be scanned."

    ^ self browseForString:aString in:(Smalltalk allClasses)

!

browseInstRefsTo:varName in:aCollectionOfClasses modificationsOnly:modsOnly title:title
    "launch a browser for all methods in aClass where the instVar named
     varName is referenced; if modsOnly is true, browse only methods where the
     instvar is modified"

    |filter browser|

    filter := self filterToSearchInstRefsTo:varName modificationsOnly:modsOnly.
    browser := self browseInstMethodsIn:aCollectionOfClasses where:filter title:title.

    browser notNil ifTrue:[
	browser autoSearch:varName 
    ].
    ^ browser

!

aproposSearch:aString in:aCollectionOfClasses
    "browse all methods, which have aString in their selector or
     in the methods comment.
     This is relatively slow, since all source must be processed."

    |matchString list s searchBlock browser|

    matchString := '*' , aString , '*'.

    list := OrderedCollection new.

    (aString includesMatchCharacters) ifTrue:[
	s := '*' , aString , '*'.
	"a matchString"
	searchBlock := [:source | s match:source]
    ] ifFalse:[
	searchBlock := [:source | (source findString:aString) ~~ 0]
    ].

    browser := self browseMethodsIn:aCollectionOfClasses 
		     where:[:class :method :sel |
				|comment|

				Processor yield.
				(searchBlock value:sel) ifTrue:[
				    true
				] ifFalse:[
				    comment := method comment.
				    comment notNil 
				    and:[searchBlock value:method comment]
				].
			   ]
		     title:('apropos: ' , aString).

    browser notNil ifTrue:[
	browser autoSearch:aString 
    ].
    ^ browser

    "
     SystemBrowser aproposSearch:'append' in:(Collection withAllSubclasses)
     SystemBrowser aproposSearch:'add'    in:(Collection withAllSubclasses)
     SystemBrowser aproposSearch:'sort'   in:(Collection withAllSubclasses)
     SystemBrowser aproposSearch:'[Aa]bsent' in:(Collection withAllSubclasses)
    "

!

aproposSearch:aString
    "browse all methods, which have aString in their selector or
     in the methods comment.
     This is relatively slow, since all source must be processed."

    ^ self aproposSearch:aString in:(Smalltalk allClasses)

!

browseInstRefsTo:aString in:aCollectionOfClasses modificationsOnly:modsOnly
    "launch a browser for all methods in aClass where the instVar named
     aString is referenced; if modsOnly is true, browse only methods where the
     instvar is modified"

    |title|

    modsOnly ifTrue:[
	title := 'modifications of '
    ] ifFalse:[
	title := 'references to '
    ].
    ^ self browseInstRefsTo:aString 
			 in:aCollectionOfClasses 
	  modificationsOnly:modsOnly 
		      title:(title , aString)

!

browseInstRefsTo:aString under:aClass modificationsOnly:modsOnly
    "launch a browser for all methods in aClass and subclasses
     where the instVar named aString is referenced; 
     if modsOnly is true, browse only methods where the instvar is modified"

    ^ self browseInstRefsTo:aString in:(aClass withAllSubclasses) modificationsOnly:modsOnly

!

browseClassRefsTo:varName in:aCollectionOfClasses modificationsOnly:modsOnly title:title
    "launch a browser for all methods in aCollectionOfClasses,
     where the classVar named aString is referenced; 
     if modsOnly is true, browse only methods where the classvar is modified"

    |searchBlock browser|

    searchBlock := self filterToSearchClassRefsTo:varName modificationsOnly:modsOnly.
    browser := self browseMethodsIn:aCollectionOfClasses where:searchBlock title:title.

    browser notNil ifTrue:[
	browser autoSearch:varName
    ].
    ^ browser

!

browseClassRefsTo:aString in:aCollectionOfClasses modificationsOnly:modsOnly
    "launch a browser for all methods in aClass where the classVar named
     aString is referenced; if modsOnly is true, browse only methods where the
     classvar is modified"

    |title|

    modsOnly ifTrue:[
	title := 'modifications of '
    ] ifFalse:[
	title := 'references to '
    ].
    ^ self browseClassRefsTo:aString in:aCollectionOfClasses modificationsOnly:modsOnly title:(title , aString)

!

browseClassRefsTo:aString under:aClass modificationsOnly:modsOnly
    "launch a browser for all methods in aClass and subclasses
     where the classVar named aString is referenced; 
     if modsOnly is true, browse only methods where the classvar is modified"

    ^ self browseClassRefsTo:aString in:(aClass withAllSubclasses) modificationsOnly:modsOnly

!

filterToSearchClassRefsTo:varName modificationsOnly:modsOnly
    "return a searchblock for class variable references"

    |searchBlock|

    searchBlock := [:c :m :s |
	|src result parser classvars needMatch|

	needMatch := varName includesMatchCharacters.

	src := m source.
	src isNil ifTrue:[
	    result := false
	] ifFalse:[
	    needMatch ifFalse:[
		"
		 before doing a slow parse, quickly scan the
		 methods source for the variables name ...
		"
		result := (src findString:varName) ~~ 0.
	    ] ifTrue:[
		result := true.
	    ].
	    result ifTrue:[
		result := false.
		parser := Parser parseMethod:src in:c warnings:false.
		parser notNil ifTrue:[
		    modsOnly ifTrue:[
			classvars := parser modifiedClassVars
		    ] ifFalse:[
			classvars := parser usedClassVars
		    ].
		    classvars notNil ifTrue:[
			needMatch ifTrue:[
			    classvars do:[:cv |
				(varName match:cv) ifTrue:[result := true]
			    ]
			] ifFalse:[
			    result := classvars includes:varName
			]
		    ]
		].
	    ].
	].
	Processor yield.
	result
    ].
    ^ searchBlock
! !

!SystemBrowser class methodsFor:'private instance creation'!

newWithLabel:aString setupBlock:aBlock on:aWorkstation
    "common helper method for all creation methods"

    |newBrowser|

    newBrowser := BrowserView on:aWorkstation.
    newBrowser title:aString.
    aBlock value:newBrowser.

    newBrowser open.
    ^ newBrowser
!

newWithLabel:aString setupBlock:aBlock
    "common helper method for all creation methods"

    ^ self newWithLabel:aString setupBlock:aBlock on:Display
! !

!SystemBrowser class methodsFor:'private helpers'!

showNoneFound:what
    self showNoneFound
!

showNoneFound
    Dialog warn:(BrowserView classResources string:'None found').
! !

SystemBrowser initialize!
